package de.intevation.flys.exports.minfo;

import org.apache.log4j.Logger;
import org.jfree.data.xy.XYSeries;
import org.w3c.dom.Document;

import de.intevation.artifactdatabase.state.ArtifactAndFacet;
import de.intevation.artifactdatabase.state.Facet;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.minfo.BedDiffYearResult;
import de.intevation.flys.exports.StyledSeriesBuilder;
import de.intevation.flys.exports.XYChartGenerator;
import de.intevation.flys.jfree.FLYSAnnotation;
import de.intevation.flys.jfree.StyledXYSeries;


public class BedDiffHeightYearGenerator
extends XYChartGenerator
implements FacetTypes
{
    public enum YAXIS {
        D(0);

        protected int idx;

        private YAXIS(int c) {
            idx = c;
        }
    }

    /** The logger that is used in this generator. */
    private static Logger logger = Logger.getLogger(BedDiffHeightYearGenerator.class);

    public static final String I18N_CHART_TITLE = "chart.beddifference.height.title";
    public static final String I18N_XAXIS_LABEL = "chart.beddifference.height.xaxis.label";
    public static final String I18N_YAXIS_LABEL = "chart.beddifference.height.yaxis.label";

    public static final String I18N_CHART_TITLE_DEFAULT = "Sohlenhöhen Differenz";
    public static final String I18N_XAXIS_LABEL_DEFAULT = "Fluss-Km";
    public static final String I18N_YAXIS_LABEL_DEFAULT = "delta S [cm / Jahr]";

    @Override
    protected YAxisWalker getYAxisWalker() {
        return new YAxisWalker() {

            @Override
            public int length() {
                return YAXIS.values().length;
            }

            @Override
            public String getId(int idx) {
                YAXIS[] yaxes = YAXIS.values();
                return yaxes[idx].toString();
            }
        };
    }

    @Override
    public void doOut(ArtifactAndFacet bundle, Document attr, boolean visible) {
        String name = bundle.getFacetName();

        logger.debug("doOut: " + name);

        if (name == null) {
            logger.error("No facet name for doOut(). No output generated!");
            return;
        }

        Facet facet = bundle.getFacet();

        if (facet == null) {
            return;
        }

        if (name.equals(BED_DIFFERENCE_HEIGHT_YEAR)) {
            doBedDifferenceYearOut(
                (BedDiffYearResult) bundle.getData(context),
                bundle, attr, visible);
        }
        else if (name.equals(LONGITUDINAL_ANNOTATION)) {
            doAnnotations(
                (FLYSAnnotation) bundle.getData(context),
                 bundle,
                 attr,
                 visible);
        }
    }

    @Override
    protected String getDefaultChartTitle() {
        return msg(I18N_CHART_TITLE, I18N_CHART_TITLE_DEFAULT);
    }

    @Override
    protected String getDefaultXAxisLabel() {
        return msg(I18N_XAXIS_LABEL, I18N_XAXIS_LABEL_DEFAULT);
    }

    @Override
    protected String getDefaultYAxisLabel(int pos) {
        return msg(I18N_YAXIS_LABEL, I18N_YAXIS_LABEL_DEFAULT);
    }

    protected void doBedDifferenceYearOut(BedDiffYearResult data,
        ArtifactAndFacet aandf, Document theme, boolean visible) {

        XYSeries series = new StyledXYSeries(aandf.getFacetDescription(), theme);
        StyledSeriesBuilder.addPoints(series, data.getHeightPerYearData(), true);

        addAxisSeries(series, YAXIS.D.idx, visible);
    }
}
